@@ -34,7 +34,7 @@ class WorkspaceRun:
 
         try:
             subprocess.run("python3 -u %s > "
-                           "/home/work/output_%s.log 2>&1"
+                           "/tmp/output_%s.log 2>&1"
                            % (self.code_path, self.workspace_run_log.workspace_run_id), shell=True, encoding='UTF-8')\
                 .check_returncode()
             self.workspace_run_log.is_success = True
